#!/usr/bin/env ruby
# by eto 2003-0112

require 'test/unit'
$LOAD_PATH << '../src'
require 'chise'
include CHISE

class TC_DB < Test::Unit::TestCase
  def setup
    @cdb = CharDB.instance
    @sdb = CodesysDB.instance
  end
  def test_db
    assert_equal("()+!", DB.unix_to_win("<>*?"))
    assert_equal("<>*?", DB.win_to_unix("()+!"))
  end
  def test_each_db(db)
    assert_instance_of(Array, db.keys)
  end
  def test_make_db(db)
    h = {'a' => 1, 'b' => 2, 'c' => 3}
    db.remove_db('test-db') #まず最初に消しておく
    assert_not_nil(db.make_db('test-db', h))
    assert_not_nil(db.open_db('test-db'))
    assert_equal(1, db.get('test-db', 'a'))
    assert_equal(2, db.get('test-db', 'b'))
    assert_equal(3, db.get('test-db', 'c'))
    db.remove_db('test-db') #最後にまた消しておく
  end
  def test_dbs
    test_each_db(@cdb)
    test_each_db(@sdb)
#    test_make_db(@cdb)
#    test_make_db(@sdb)
  end
  def test_db_put
    char = "字".char
    char.put_char_attribute('test-attribute', 'test')
    assert_equal('test', char.test_attribute)
  end
end

class TC_Codesys < Test::Unit::TestCase
  def setup
    @db = CodesysDB.instance
  end
  def nu_test_db_length
    assert_equal(6287, @db.get('=jis-x0208').keys.length, "keys")
    assert_equal(590,  @db.get('japanese-jisx0208').keys.length, "keys")
    assert_equal(499,  @db.get('japanese-jisx0208-1978').keys.length, "keys")
    assert_equal(593,  @db.get('japanese-jisx0208-1990').keys.length, "keys")
    assert_equal(6067, @db.get('japanese-jisx0212').keys.length, "keys")
    assert_equal(1697, @db.get('japanese-jisx0213-1').keys.length, "keys")
    assert_equal(2345, @db.get('japanese-jisx0213-2').keys.length, "keys")
    assert_equal(4270, @db.get('ucs-jis').keys.length, "keys")
  end
  def test_db
    keys = @db.keys
    assert_instance_of(Array, @db.keys, "db.keys")
    db = @db.get('ascii')
    assert_equal(128, db.keys.length, "can get keys")
    assert_equal(63,   @db.get('katakana-jisx0201').keys.length, "keys")
    assert_equal(94,   @db.get('latin-jisx0201').keys.length, "keys")

    counter = 0
    @db.each('=jis-x0208'){|k, v| #引数のCodesysデータベースのそれぞれに対して実行する
      er0 = sprintf("&J90-%04X;", k)
      er1 = Character.new(v).to_er
      counter += 1; break if 10 < counter
    }
  end
  def test_ascii
    db = CodesysDB.instance
    codesys = db.get_codesys('ascii')
    char = codesys.get(65)
    assert_equal("A", char.to_s)
    assert_equal(128, codesys.keys.length)
    ks = codesys.keys
  end
  def test_jis_codesys
    db = CodesysDB.instance
    codesys = db.get_codesys('=jis-x0208')
    ks = codesys.keys.sort #とすることによって、JISX0208 1990の集合全部のkeysが得られる
#    assert_equal(6880, ks.length)
    assert_equal(8481, ks.first)
    assert_equal(29566, ks.last)
    char = codesys.get(15226) #"字"
    assert_equal("字", char.to_s)

    assert_equal("亜", codesys.get(12321))
    jis = "亜".char.japanese_jisx0208_1990
#    assert_equal("亜", codesys.get(jis))
#    assert_equal("亜", sprintf("&J90-%04X;", jis).de_er)

#    codesys = db.get_codesys('japanese-jisx0208-1990') #旧名
    codesys = db.get_codesys('=jis-x0208-1990')
    assert_equal(8481, ks.first)
    assert_equal(29566, ks.last)
  end
end

#----------------------------------------------------------------------end.
